/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2009 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* File      slotcar.h
*
* Author    Milan Brejl
* 
* Version   1.0
* 
* Date      16-Nov-2009
* 
* Brief     Slot car HW related macros and routines
*
*******************************************************************************
* History:
* 1.0 (26-Nov-2009) Initial version
******************************************************************************/
#ifndef _SLOTCAR_H_
#define _SLOTCAR_H_

#include "derivative.h"     /* include peripheral declarations */

/******************************************************************************
* Constants and macros
******************************************************************************/

/* LEDs */
/* 
 * There are 4 LEDs build into the head and break lights.
 *   HL = head light - left ..... CON4
 *   BL = break light - left .... CON5
 *   HR = head light - right .... CON6
 *   BR = break light - right ... CON7
 * The following macros control the LEDs state, ON or OFF.
 */
#define SET_LED_HL_ON          PTFD_PTFD0 = 1
#define SET_LED_HL_OFF         PTFD_PTFD0 = 0
#define SET_LED_BL_ON          PTFD_PTFD1 = 1
#define SET_LED_BL_OFF         PTFD_PTFD1 = 0
#define SET_LED_HR_ON          PTFD_PTFD2 = 1
#define SET_LED_HR_OFF         PTFD_PTFD2 = 0
#define SET_LED_BR_ON          PTFD_PTFD3 = 1
#define SET_LED_BR_OFF         PTFD_PTFD3 = 0

/* SWITCH */
/*
 * There is a jumper switch of 2 positions.
 * The following macros read the switch position.
 */
#define GET_SWITCH_STATE       PTAD_PTAD5
#define SWITCH_POSITION_1_2    (GET_SWITCH_STATE == 0)
#define SWITCH_POSITION_2_3    (GET_SWITCH_STATE == 1)

/* ACCELEROMETER */
/*
 * There is a 3-axis accelerometer with analog outputs.
 * The accelerometer range is fixed to range +/- 1.5g
 * The accelerometer analog outputs can be sampled by the on-chip Analog-to-
 * -Digital Converter.
 * The following macros define the ADC channels of the X, Y and Z accelerometer 
 * outputs:
 *   X ... left and right
 *   Y ... ahead and back
 *   Z ... up and down
 */
#define ACC_X                  8
#define ACC_Y                  7
#define ACC_Z                  6

/* MOTOR */
/*
 * There is an H-brigde driving the DC motor.
 * The motor drive can be enabled or disabled. When enabled, the applied motor
 * voltage is controlled by a PWM duty-cycle.
 * The H-brigde circuit returns a fault status in case of undervoltage,
 * short circuit, or overtemperature condition.
 * The motor current feedback can be sampled by the on-chip Analog-to-Digital
 * Converter.
 * The following macros enable or disable the motor drive, set the motor 
 * voltage, get the motor fault status and define the ADC channel of
 * the motor current.
 */
#define MOTOR_ENABLE           PTCD_PTCD6 = 1
#define MOTOR_DISABLE          PTCD_PTCD6 = 0
#define SET_MOTOR_VOLTAGE(x)   TPM2C1V = 6000 - (x)
#define GET_MOTOR_VOLTAGE      (6000 - TPM2C1V)
#define GET_MOTOR_FAULT_STATUS PTGD_PTGD1
#define MOTOR_STATUS_OK        1
#define MOTOR_STATUS_FAULT     0
#define MOTOR_CURRENT          10

/* TRACK VOLTAGE AND POWER OUTAGE */
/*
 * A DC-Bus Voltage Compensation technique enables to set a correct motor 
 * voltage and keep it independent of a the actual track voltage.
 * The voltage on both track lines can be measured from the INT1 and INT2 
 * inputs. The following macros define the ADC channels of INT1 and INT2.
 */
#define TRACK_VOLTAGE_1        4
#define TRACK_VOLTAGE_2        5
/*
 * INT1 and INT2 inputs could be also used for power supply outage
 * detection on the changeover track pieces using KBI. The following
 * macros define the INT1 and INT2 input pin state. These macros can
 * not be used if INT1 and INT2 are ADC inputs.
 */
#define GET_INT1               PTBD_PTBD4
#define GET_INT2               PTBD_PTBD5
/*
 * The following macros enables to switch pin function of INT1 and INT2
 * between KBI/GPIO and ADC.
 */
#define INT1_TO_ADC_CONNECT    (APCTL1_ADPC4 = 1)
#define INT2_TO_ADC_CONNECT    (APCTL1_ADPC5 = 1)
#define INTx_TO_ADC_DISCONNECT (APCTL1 &= ~(APCTL1_ADPC4_MASK | APCTL1_ADPC5_MASK))

/* ADC */
/*
 * The following macros enable to initializes the sampling and 
 * conversion of an analog signal a defined read a conversion result - a 12-bit sample,
 * to check if the Analog-to-Digital Converter is busy by an active conversion, and
 * switch conversion resolution (8/10/12-bit mode)
 */
#define START_CONV(channel)    ADCSC1 = ADCSC1_AIEN_MASK | (channel)
#define READ_ADC_SAMPLE        ADCR
#define GET_ADC_ACTIVE_FLAG    ADCSC2_ADACT
#define SET_ADC_MODE_8_BIT     ADCCFG_MODE = 0
#define SET_ADC_MODE_10_BIT    ADCCFG_MODE = 2
#define SET_ADC_MODE_12_BIT    ADCCFG_MODE = 1


/**************************** Function Prototypes *****************************/          
void SlotCarInit(void);



#endif /* _SLOTCAR_H_ */

/*****************************************************************************/                                                                        
/* Freescale  is  not  obligated  to  provide  any  support, upgrades or new */
/* releases  of  the Software. Freescale may make changes to the Software at */
/* any time, without any obligation to notify or provide updated versions of */
/* the  Software  to you. Freescale expressly disclaims any warranty for the */
/* Software.  The  Software is provided as is, without warranty of any kind, */
/* either  express  or  implied,  including, without limitation, the implied */
/* warranties  of  merchantability,  fitness  for  a  particular purpose, or */
/* non-infringement.  You  assume  the entire risk arising out of the use or */
/* performance of the Software, or any systems you design using the software */
/* (if  any).  Nothing  may  be construed as a warranty or representation by */
/* Freescale  that  the  Software  or  any derivative work developed with or */
/* incorporating  the  Software  will  be  free  from  infringement  of  the */
/* intellectual property rights of third parties. In no event will Freescale */
/* be  liable,  whether in contract, tort, or otherwise, for any incidental, */
/* special,  indirect, consequential or punitive damages, including, but not */
/* limited  to,  damages  for  any loss of use, loss of time, inconvenience, */
/* commercial loss, or lost profits, savings, or revenues to the full extent */
/* such  may be disclaimed by law. The Software is not fault tolerant and is */
/* not  designed,  manufactured  or  intended by Freescale for incorporation */
/* into  products intended for use or resale in on-line control equipment in */
/* hazardous, dangerous to life or potentially life-threatening environments */
/* requiring  fail-safe  performance,  such  as  in the operation of nuclear */
/* facilities,  aircraft  navigation  or  communication systems, air traffic */
/* control,  direct  life  support machines or weapons systems, in which the */
/* failure  of  products  could  lead  directly to death, personal injury or */
/* severe  physical  or  environmental  damage  (High  Risk Activities). You */
/* specifically  represent and warrant that you will not use the Software or */
/* any  derivative  work of the Software for High Risk Activities.           */
/* Freescale  and the Freescale logos are registered trademarks of Freescale */
/* Semiconductor Inc.                                                        */
/*****************************************************************************/